@@ -66,6 +66,24 @@ static lua_State *luaState = 0;
 static bool luaDisabled = false;
 
 static char *startupScript = NULL;
+static SString extensionScript;
+
+static bool tracebackEnabled = true;
+
+static int maxBufferIndex = -1;
+static int curBufferIndex = -1;
+
+static int GetPropertyInt(const char *propName) {
+	int propVal = 0;
+	if (host) {
+		char *pszPropVal = host->Property(propName);
+		if (pszPropVal) {
+			propVal = atoi(pszPropVal);
+			delete [] pszPropVal;
+		}
+	}
+	return propVal;
+}
 
 LuaExtension::LuaExtension() {}
 
@@ -76,6 +94,29 @@ LuaExtension &LuaExtension::Instance() {
 	return singleton;
 }
 
+// Forward declarations
+static ExtensionAPI::Pane check_pane_object(lua_State *L, int index);
+static void push_pane_object(lua_State *L, ExtensionAPI::Pane p);
+static int iface_function_helper(lua_State *L, const IFaceFunction &func);
+
+inline bool IFaceTypeIsScriptable(IFaceType t, int index) {
+	return t < iface_stringresult || (index==1 && t == iface_stringresult);
+}
+
+inline bool IFaceTypeIsNumeric(IFaceType t) {
+	return (t > iface_void && t < iface_bool);
+}
+
+inline bool IFaceFunctionIsScriptable(const IFaceFunction &f) {
+	return IFaceTypeIsScriptable(f.paramType[0], 0) && IFaceTypeIsScriptable(f.paramType[1], 1);
+}
+
+inline bool IFacePropertyIsScriptable(const IFaceProperty &p) {
+	return (((p.valueType > iface_void) && (p.valueType <= iface_string) && (p.valueType != iface_keymod)) &&
+		((p.paramType < iface_colour) || (p.paramType == iface_string) ||
+		(p.paramType == iface_bool)) && (p.getter || p.setter));
+}
+
 static int cf_getglobal_wrapper(lua_State *L) {
 	if (lua_gettop(L) == 1) {
 		lua_gettable(L, LUA_GLOBALSINDEX);
@@ -113,16 +154,21 @@ inline int absolute_index(lua_State *L, int index) {
 		? (lua_gettop(L) + index + 1) : index;
 }
 
-static int merge_table(lua_State *L, int destTableIdx, int srcTableIdx) {
+static int merge_table(lua_State *L, int destTableIdx, int srcTableIdx, bool copyMetatable = false) {
 	int count = 0;
 	if (lua_istable(L, destTableIdx) && lua_istable(L, srcTableIdx)) {
 		srcTableIdx = absolute_index(L, srcTableIdx);
 		destTableIdx = absolute_index(L, destTableIdx);
+		if (copyMetatable) {
+			lua_getmetatable(L, srcTableIdx);
+			lua_setmetatable(L, destTableIdx);
+		}
 
-		lua_pushnil(L); /* first key */
+		lua_pushnil(L); // first key
 		while (lua_next(L, srcTableIdx) != 0) {
-			/* key is at index -2 and value at index -1 */
-			lua_pushvalue(L, -2);
+			// key is at index -2 and value at index -1
+			lua_pushvalue(L, -2); // the key
+			lua_insert(L, -2); // leaving value (-1), key (-2), key (-3)
 			lua_rawset(L, destTableIdx);
 			++count;
 		}
@@ -134,11 +180,7 @@ static bool clone_table(lua_State *L, int srcTableIdx, bool copyMetatable = fals
 	if (lua_istable(L, srcTableIdx)) {
 		srcTableIdx = absolute_index(L, srcTableIdx);
 		lua_newtable(L);
-		if (copyMetatable) {
-			lua_getmetatable(L, srcTableIdx);
-			lua_setmetatable(L, -2);
-		}
-		merge_table(L, -1, srcTableIdx);
+		merge_table(L, -1, srcTableIdx, copyMetatable);
 		return true;
 	} else {
 		lua_pushnil(L);
@@ -146,6 +188,24 @@ static bool clone_table(lua_State *L, int srcTableIdx, bool copyMetatable = fals
 	}
 }
 
+static void clear_table(lua_State *L, int tableIdx, bool clearMetatable = true) {
+	if (lua_istable(L, tableIdx)) {
+		tableIdx = absolute_index(L, tableIdx);
+	if (clearMetatable) {
+		lua_pushnil(L);
+		lua_setmetatable(L, tableIdx);
+	}
+		lua_pushnil(L); // first key
+		while (lua_next(L, tableIdx) != 0) {
+			// key is at index -2 and value at index -1
+			lua_pop(L, 1); // discard value
+			lua_pushnil(L); 
+			lua_rawset(L, tableIdx); // table[key] = nil
+			lua_pushnil(L); // get 'new' first key
+		}
+	}
+}
+
 static bool rawget_library_object(lua_State *L, const char *basename, const char *subname = NULL) {
 	lua_pushstring(L, basename);
 	lua_rawget(L, LUA_GLOBALSINDEX);
@@ -165,14 +225,80 @@ static bool rawget_library_object(lua_State *L, const char *basename, const char
 	return false;
 }
 
+static int cf_scite_send(lua_State *L) {
+	// This is reinstated as a replacement for the old <pane>:send, which was removed
+	// due to safety concerns.  Is now exposed as scite.SendEditor / scite.SendOutput.
+	// It is rewritten to be typesafe, checking the arguments against the metadata in
+	// IFaceTable in the same way that the object interface does.
 
-static void push_pane_object(lua_State *L, ExtensionAPI::Pane p);
+	int paneIndex = lua_upvalueindex(1);
+	check_pane_object(L, paneIndex);
+	int message = luaL_checkint(L, 1);
+
+	lua_pushvalue(L, paneIndex);
+	lua_replace(L, 1);
+
+	IFaceFunction func = { "", 0, iface_void, {iface_void, iface_void} };
+	for (int funcIdx = 0; funcIdx < IFaceTable::functionCount; ++funcIdx) {
+		if (IFaceTable::functions[funcIdx].value == message) {
+			func = IFaceTable::functions[funcIdx];
+			break;
+		}
+	}
+
+	if (func.value == 0) {
+		for (int propIdx = 0; propIdx < IFaceTable::propertyCount; ++propIdx) {
+			if (IFaceTable::properties[propIdx].getter == message) {
+				func = IFaceTable::properties[propIdx].GetterFunction();
+				break;
+			} else if (IFaceTable::properties[propIdx].setter == message) {
+				func = IFaceTable::properties[propIdx].SetterFunction();
+				break;
+			}
+		}
+	}
+
+	if (func.value != 0) {
+		if (IFaceFunctionIsScriptable(func)) {
+			return iface_function_helper(L, func);
+		} else {
+			raise_error(L, "Cannot call send for this function: not scriptable.");
+			return 0;
+		}
+	} else {
+		raise_error(L, "Message number does not match any published Scintilla function or property");
+		return 0;
+	}
+}
+
+static int cf_scite_constname(lua_State *L) {
+	char constName[100] = "";
+	int message = luaL_checkint(L, 1);
+	if (IFaceTable::GetConstantName(message, constName, 100) > 0) {
+		lua_pushstring(L, constName);
+		return 1;
+	} else {
+		raise_error(L, "Argument does not match any Scintilla / SciTE constant");
+		return 0;
+	}
+}
+
+static int cf_scite_open(lua_State *L) {
+	const char *s = luaL_checkstring(L, 1);
+	if (s) {
+		SString cmd = "open:";
+		cmd += s;
+		cmd.substitute("\\", "\\\\");
+		host->Perform(cmd.c_str());
+	}
+	return 0;
+}
 
 
 static ExtensionAPI::Pane check_pane_object(lua_State *L, int index) {
 	ExtensionAPI::Pane *pPane = reinterpret_cast<ExtensionAPI::Pane*>(luaL_checkudata(L, index, "SciTE_MT_Pane"));
 
-	if (!pPane) {
+	if ((!pPane) && lua_istable(L, index)) {
 		// so that nested objects have a convenient way to do a back reference
 		int absIndex = absolute_index(L, index);
 		lua_pushliteral(L, "pane");
@@ -180,11 +306,17 @@ static ExtensionAPI::Pane check_pane_object(lua_State *L, int index) {
 		pPane = reinterpret_cast<ExtensionAPI::Pane*>(luaL_checkudata(L, -1, "SciTE_MT_Pane"));
 	}
 
-	if (pPane)
+	if (pPane) {
+		if ((*pPane == ExtensionAPI::paneEditor) && (curBufferIndex < 0))
+			raise_error(L, "Editor pane is not accessible at this time.");
+
 		return *pPane;
+	}
 
 	if (index == 1)
 		lua_pushliteral(L, "Self object is missing in pane method or property access.");
+	else if (index == lua_upvalueindex(1))
+		lua_pushliteral(L, "Internal error: pane object expected in closure.");
 	else
 		lua_pushliteral(L, "Pane object expected.");
 
@@ -509,16 +641,36 @@ static int cf_props_metatable_newindex(lua_State *L) {
 	int selfArg = lua_isuserdata(L, 1) ? 1 : 0;
 
 	const char *key = lua_isstring(L, selfArg + 1) ? lua_tostring(L, selfArg + 1) : 0;
-	const char *val = luaL_checkstring(L, selfArg + 2);
+	const char *val = lua_tostring(L, selfArg + 2);
 
-	if (key && *key && val) {
-		host->SetProperty(key, val);
+	if (key && *key) {
+		if (val) {
+			host->SetProperty(key, val);
+		} else if (lua_isnil(L, selfArg + 2)) {
+			host->UnsetProperty(key);
+		} else {
+			raise_error(L, "Expected string or nil for property assignment.");
+		}
 	} else {
-		raise_error(L, "Property name and value must be strings");
+		raise_error(L, "Property name must be a non-empty string.");
 	}
 	return 0;
 }
 
+/*
+static int cf_os_execute(lua_State *L) {
+	// The SciTE version of os.execute would pipe its stdout and stderr
+	// to the output pane.  This can be implemented in terms of popen
+	// on GTK and in terms of CreateProcess on Windows.  Either way,
+	// stdin should be null, and the Lua script should wait for the
+	// subprocess to finish before continuing.  (What if it takes
+	// a very long time?  Timeout?)
+
+	raise_error(L, "Not implemented.");
+	return 0;
+}
+*/
+
 static int cf_global_print(lua_State *L) {
 	int nargs = lua_gettop(L);
 
@@ -583,28 +735,22 @@ static int cf_global_alert(lua_State *L) {
 
 
 static int cf_global_dostring(lua_State *L) {
-	if (host) {
-		if (rawget_library_object(L, "loadstring")) {
-			lua_insert(L, 1);
-			lua_call(L, lua_gettop(L)-1, 2);
-			if (lua_isnil(L, -1)) {
-				lua_pop(L, 1);
-				lua_call(L, 0, 0);
-			} else {
-				lua_remove(L, -2);
-				raise_error(L);
-				// lua_error(L);
-			}
-		} else {
-			raise_error(L, "Internal error - missing required library function loadstring");
-		}
+	int nargs = lua_gettop(L);
+	const char *code = luaL_checkstring(L, 1);
+	const char *name = luaL_optstring(L, 2, code);
+	if (0 == luaL_loadbuffer(L, code, lua_strlen(L, 1), name)) {
+		lua_call(L, 0, LUA_MULTRET);
+		return lua_gettop(L) - nargs;
+	} else {
+		raise_error(L);
 	}
 	return 0;
 }
 
 static void call_alert(lua_State *L, const char *s = NULL) {
 	lua_pushliteral(L, "_ALERT");
-	if (!safe_getglobal(L) || !lua_isfunction(L, -1)) {
+	lua_rawget(L, LUA_GLOBALSINDEX);
+	if (!lua_isfunction(L, -1)) {
 		lua_pop(L, 1);
 		lua_pushcfunction(L, cf_global_print);
 	}
@@ -621,7 +767,23 @@ static bool call_function(lua_State *L, int nargs, bool ignoreFunctionReturnValu
 	bool handled = false;
 
 	if (L) {
-		int result = lua_pcall(L, nargs, ignoreFunctionReturnValue ? 0 : 1, 0);
+		int traceback = 0;
+		if (tracebackEnabled) {
+			lua_pushliteral(L, "_TRACEBACK");
+			if (safe_getglobal(L) && lua_isfunction(L, -1)) {
+				traceback = lua_gettop(L) - nargs - 1;
+				lua_insert(L, traceback);
+			} else {
+				lua_pop(L, 1);
+			}
+		}
+
+		int result = lua_pcall(L, nargs, ignoreFunctionReturnValue ? 0 : 1, traceback);
+
+		if (traceback) {
+			lua_remove(L, traceback);
+		}
+
 		if (0 == result) {
 			if (ignoreFunctionReturnValue) {
 				handled = true;
@@ -636,7 +798,7 @@ static bool call_function(lua_State *L, int nargs, bool ignoreFunctionReturnValu
 			if (result == LUA_ERRMEM) {
 				host->Trace("> Lua: memory allocation error\n");
 			} else if (result == LUA_ERRERR) {
-				host->Trace("> Lua: error function failed\n");
+				host->Trace("> Lua: an error occurred, but cannot be reported due to failure in _TRACEBACK\n");
 			} else {
 				host->Trace("> Lua: unexpected error\n");
 			}
@@ -668,32 +830,25 @@ static bool CallNamedFunction(const char *name, const char *arg) {
 	return handled;
 }
 
-
-inline bool IFaceTypeIsScriptable(IFaceType t, int index) {
-	return t < iface_stringresult || (index==1 && t == iface_stringresult);
+static bool CallNamedFunction(const char *name, int numberArg, const char *stringArg) {
+	bool handled = false;
+	if (luaState) {
+		lua_pushstring(luaState, name);
+		if (safe_getglobal(luaState) && lua_isfunction(luaState, -1)) {
+			lua_pushnumber(luaState, numberArg);
+			lua_pushstring(luaState, stringArg);
+			handled = call_function(luaState, 2);
+		}
+	}
+	return handled;
 }
 
-inline bool IFaceTypeIsNumeric(IFaceType t) {
-	return (t > iface_void && t < iface_bool);
-}
-
-inline bool IFaceFunctionIsScriptable(const IFaceFunction &f) {
-	return IFaceTypeIsScriptable(f.paramType[0], 0) && IFaceTypeIsScriptable(f.paramType[1], 1);
-}
-
-inline bool IFacePropertyIsScriptable(const IFaceProperty &p) {
-	return (((p.valueType > iface_void) && (p.valueType < iface_keymod)) &&
-		((p.paramType < iface_colour) || (p.paramType == iface_string) ||
-		(p.paramType == iface_bool)) && (p.getter || p.setter));
-}
-
-
 static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	ExtensionAPI::Pane p = check_pane_object(L, 1);
 
 	int arg = 2;
 
-	int params[2] = {0,0};
+	long params[2] = {0,0};
 
 	char *stringResult = 0;
 	bool needStringResult = false;
@@ -701,8 +856,8 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	int loopParamCount = 2;
 
 	if (func.paramType[0] == iface_length && func.paramType[1] == iface_string) {
-		params[0] = static_cast<int>(lua_strlen(L, arg));
-		params[1] = reinterpret_cast<int>(params[0] ? lua_tostring(L, arg) : "");
+		params[0] = static_cast<long>(lua_strlen(L, arg));
+		params[1] = reinterpret_cast<long>(params[0] ? lua_tostring(L, arg) : "");
 		loopParamCount = 0;
 	} else if (func.paramType[1] == iface_stringresult) {
 		needStringResult = true;
@@ -718,7 +873,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	for (int i=0; i<loopParamCount; ++i) {
 		if (func.paramType[i] == iface_string) {
 			const char *s = lua_tostring(L, arg++);
-			params[i] = reinterpret_cast<int>(s ? s : "");
+			params[i] = reinterpret_cast<long>(s ? s : "");
 		} else if (func.paramType[i] == iface_keymod) {
 			int keycode = static_cast<int>(luaL_checknumber(L, arg++)) & 0xFFFF;
 			int modifiers = static_cast<int>(luaL_checknumber(L, arg++)) & (SCMOD_SHIFT|SCMOD_CTRL|SCMOD_ALT);
@@ -726,7 +881,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 		} else if (func.paramType[i] == iface_bool) {
 			params[i] = lua_toboolean(L, arg++);
 		} else if (IFaceTypeIsNumeric(func.paramType[i])) {
-			params[i] = static_cast<int>(luaL_checknumber(L, arg++));
+			params[i] = static_cast<long>(luaL_checknumber(L, arg++));
 		}
 	}
 
@@ -738,7 +893,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 			stringResult = new char[stringResultLen+1];
 			if (stringResult) {
 				stringResult[stringResultLen]='\0';
-				params[1] = reinterpret_cast<int>(stringResult);
+				params[1] = reinterpret_cast<long>(stringResult);
 			} else {
 				raise_error(L, "String result buffer allocation failed");
 				return 0;
@@ -829,10 +984,9 @@ static int cf_ifaceprop_metatable_newindex(lua_State *L) {
 
 static int cf_pane_iface_function(lua_State *L) {
 	int funcidx = lua_upvalueindex(1);
-
-	if (lua_islightuserdata(L, funcidx)) {
-		const IFaceFunction &func = *reinterpret_cast<IFaceFunction *>(lua_touserdata(L, funcidx));
-		return iface_function_helper(L, func);
+	const IFaceFunction *func = reinterpret_cast<IFaceFunction *>(lua_touserdata(L, funcidx));
+	if (func) {
+		return iface_function_helper(L, *func);
 	} else {
 		raise_error(L, "Internal error - bad upvalue in iface function closure");
 		return 0;
@@ -845,6 +999,12 @@ static int push_iface_function(lua_State *L, const char *name) {
 		if (IFaceFunctionIsScriptable(IFaceTable::functions[i])) {
 			lua_pushlightuserdata(L, const_cast<IFaceFunction*>(IFaceTable::functions+i));
 			lua_pushcclosure(L, cf_pane_iface_function, 1);
+
+			// Since Lua experts say it is inefficient to create closures / cfunctions
+			// in an inner loop, I tried caching the closures in the metatable, and looking
+			// for them there first.  However, it made very little difference and did not
+			// seem worth the added complexity. - WBD
+
 			return 1;
 		}
 	}
@@ -857,8 +1017,10 @@ static int push_iface_propval(lua_State *L, const char *name) {
 	int propidx = IFaceTable::FindProperty(name);
 	if (propidx >= 0) {
 		const IFaceProperty &prop = IFaceTable::properties[propidx];
-		if (!IFacePropertyIsScriptable(prop))
-			return 0;
+		if (!IFacePropertyIsScriptable(prop)) {
+			raise_error(L, "Error: iface property is not scriptable.");
+			return -1;
+		}
 
 		if (prop.paramType == iface_void) {
 			if (prop.getter) {
@@ -911,24 +1073,6 @@ static int push_iface_propval(lua_State *L, const char *name) {
 	return -1; // signal to try next pane index handler
 }
 
-
-/*
-static int push_named_function(lua_State *L, NamedFunction *namedFunctions, const char *name, int closureStackIndex=0) {
-	for (int i=0; namedFunctions[i].name; ++i) {
-		if (0==strcmp(name, namedFunctions[i].name)) {
-			if (closureStackIndex && lua_type(L, closureStackIndex) != LUA_TNONE) {
-				lua_pushvalue(L, closureStackIndex);
-				lua_pushcclosure(L, namedFunctions[i].func, 1);
-			} else {
-				lua_pushcfunction(L, namedFunctions[i].func);
-			}
-			return 1;
-		}
-	}
-	return -1;
-}
-*/
-
 static int cf_pane_metatable_index(lua_State *L) {
 	if (lua_isstring(L, 2)) {
 		const char *name = lua_tostring(L, 2);
@@ -1035,6 +1179,11 @@ void push_pane_object(lua_State *L, ExtensionAPI::Pane p) {
 static int cf_global_metatable_index(lua_State *L) {
 	if (lua_isstring(L, 2)) {
 		const char *name = lua_tostring(L, 2);
+		if ((name[0] < 'A') || (name[0] > 'Z') || ((name[1] >= 'a') && (name[1] <= 'z'))) {
+			// short circuit; iface constants are always upper-case and start with a letter
+			return 0;
+		}
+
 		int i = IFaceTable::FindConstant(name);
 		if (i >= 0) {
 			lua_pushnumber(L, IFaceTable::constants[i].value);
@@ -1043,6 +1192,14 @@ static int cf_global_metatable_index(lua_State *L) {
 			i = IFaceTable::FindFunctionByConstantName(name);
 			if (i >= 0) {
 				lua_pushnumber(L, IFaceTable::functions[i].value);
+
+				// FindFunctionByConstantName is slow, so cache the result into the
+				// global table.  My tests show this gives an order of magnitude
+				// improvement.
+				lua_pushvalue(L, 2);
+				lua_pushvalue(L, -2);
+				lua_rawset(L, 1);
+
 				return 1;
 			}
 		}
@@ -1087,24 +1244,65 @@ static char *CheckStartupScript() {
 	return startupScript;
 }
 
-static inline int CheckResetMode() {
-	int resetMode = 0;
-	char *pszResetMode = host->Property("ext.lua.reset");
-	if (pszResetMode) {
-		resetMode = atoi(pszResetMode);
-		delete [] pszResetMode;
+void IsolateGlobalReqLoadedTable() {
+	// The require built-in uses a global _LOADED to remember which
+	// files have been loaded.  Since we're resetting the global
+	// scope in Extension::Clear, the _LOADED table should be reset
+	// to its original value.  To accomplish this, while still
+	// remembering any files that were loaded in startup, a deep
+	// copy of the _LOADED table is needed.
+
+	lua_pushliteral(luaState, "_LOADED");
+	lua_rawget(luaState, LUA_GLOBALSINDEX);
+	if (!lua_isnil(luaState, -1)) {
+		lua_pushliteral(luaState, "_LOADED");
+		clone_table(luaState, -2);
+		lua_rawset(luaState, LUA_GLOBALSINDEX);
 	}
-	return resetMode;
+	lua_pop(luaState, 1);
 }
 
+void PublishGlobalBufferData() {
+	lua_pushliteral(luaState, "buffer");
+	if (curBufferIndex >= 0) {
+		lua_pushliteral(luaState, "SciTE_BufferData_Array");
+		lua_rawget(luaState, LUA_REGISTRYINDEX);
+		if (!lua_istable(luaState, -1)) {
+			lua_pop(luaState, 1);
 
-static bool InitGlobalScope(bool checkProperties) {
-	int resetMode = checkProperties ? CheckResetMode() : 0;
-
-	if (luaState && resetMode==2) {
-		lua_close(luaState);
-		luaState = 0;
+			lua_newtable(luaState);
+			lua_pushliteral(luaState, "SciTE_BufferData_Array");
+			lua_pushvalue(luaState, -2);
+			lua_rawset(luaState, LUA_REGISTRYINDEX);
+		}
+		lua_rawgeti(luaState, -1, curBufferIndex);
+		if (!lua_istable(luaState, -1)) {
+			// create new buffer-data
+			lua_pop(luaState, 1);
+			lua_newtable(luaState);
+			// remember it
+			lua_pushvalue(luaState, -1);
+			lua_rawseti(luaState, -3, curBufferIndex);
+		}
+		// Replace SciTE_BufferData_Array in the stack, leaving (buffer=-1, 'buffer'=-2)
+		lua_replace(luaState, -2);
+	} else {
+		// for example, during startup, before any InitBuffer / ActivateBuffer
+		lua_pushnil(luaState);
 	}
+	lua_rawset(luaState, LUA_GLOBALSINDEX);
+}
+
+static bool InitGlobalScope(bool checkProperties, bool forceReload = false) {
+	bool reload = forceReload;
+	if (checkProperties) {
+		int resetMode = GetPropertyInt("ext.lua.reset");
+		if (resetMode >= 1) {
+			reload = true;
+		}
+	}
+
+	tracebackEnabled = (GetPropertyInt("ext.lua.debug.traceback") == 1);
 
 	if (luaState) {
 		// The Clear / Load used to use metatables to setup without having to re-run the scripts,
@@ -1112,25 +1310,39 @@ static bool InitGlobalScope(bool checkProperties) {
 		// rawget to access functions in the global scope.  So the new method makes a shallow
 		// copy of the initialized global environment, and uses that to re-init the scope.
 
-		if (resetMode==0) {
+		if (!reload) {
 			lua_pushliteral(luaState, "SciTE_InitialState");
 			lua_rawget(luaState, LUA_REGISTRYINDEX);
 			if (lua_istable(luaState, -1)) {
-				clone_table(luaState, -1, true);
-				lua_replace(luaState, LUA_GLOBALSINDEX);
+				clear_table(luaState, LUA_GLOBALSINDEX, true);
+				merge_table(luaState, LUA_GLOBALSINDEX, -1, true);
+				lua_pop(luaState, 1);
+
+				IsolateGlobalReqLoadedTable();
+				PublishGlobalBufferData();
+
 				return true;
+			} else {
+				lua_pop(luaState, 1);
 			}
 		}
 
-		// ext.lua.reset is enabled, or else the inital state has been broken.
+		// reload mode is enabled, or else the inital state has been broken.
 		// either way, we're going to need a "new" initial state.
 
 		lua_pushliteral(luaState, "SciTE_InitialState");
 		lua_pushnil(luaState);
 		lua_rawset(luaState, LUA_REGISTRYINDEX);
 
-		lua_newtable(luaState);
-		lua_replace(luaState, LUA_GLOBALSINDEX);
+		// Also reset buffer data, since scripts might depend on this to know
+		// whether they need to re-initialize something.
+		lua_pushliteral(luaState, "SciTE_BufferData_Array");
+		lua_pushnil(luaState);
+		lua_rawset(luaState, LUA_REGISTRYINDEX);
+
+		// Don't replace global scope using new_table, because then startup script is
+		// bound to a different copy of the globals than the extension script.
+		clear_table(luaState, LUA_GLOBALSINDEX, true);
 	} else if (!luaDisabled) {
 		luaState = lua_open();
 		if (!luaState) {
@@ -1143,6 +1355,8 @@ static bool InitGlobalScope(bool checkProperties) {
 		return false;
 	}
 
+	int stackBase = lua_gettop(luaState);
+
 	// ...register standard libraries
 	luaopen_base(luaState);
 	luaopen_string(luaState);
@@ -1150,6 +1364,10 @@ static bool InitGlobalScope(bool checkProperties) {
 	luaopen_math(luaState);
 	luaopen_io(luaState);
 	luaopen_debug(luaState);
+#if PLAT_WIN
+	// loadlib might also work on Linux and some Unix variants, with some additional defines.
+	luaopen_loadlib(luaState);
+#endif
 
 	lua_register(luaState, "_ALERT", cf_global_print);
 
@@ -1166,6 +1384,9 @@ static bool InitGlobalScope(bool checkProperties) {
 	// override a library function whose default impl uses stdout
 	lua_register(luaState, "print", cf_global_print);
 
+	// the above calls all leave something on the stack: the library table, the function, etc.
+	lua_settop(luaState, stackBase);
+
 	// props object - provides access to Property and SetProperty
 	lua_pushliteral(luaState, "props");
 	lua_newuserdata(luaState, 1); // the value doesn't matter.
@@ -1181,13 +1402,38 @@ static bool InitGlobalScope(bool checkProperties) {
 	lua_rawset(luaState, LUA_GLOBALSINDEX);
 
 	// pane objects
-	lua_pushstring(luaState, "editor");
+	lua_pushliteral(luaState, "editor");
 	push_pane_object(luaState, ExtensionAPI::paneEditor);
-	lua_settable(luaState, LUA_GLOBALSINDEX);
+	lua_rawset(luaState, LUA_GLOBALSINDEX);
 
-	lua_pushstring(luaState, "output");
+	lua_pushliteral(luaState, "output");
 	push_pane_object(luaState, ExtensionAPI::paneOutput);
-	lua_settable(luaState, LUA_GLOBALSINDEX);
+	lua_rawset(luaState, LUA_GLOBALSINDEX);
+
+	lua_pushliteral(luaState, "scite");
+	lua_newtable(luaState);
+	
+	lua_pushliteral(luaState, "SendEditor");
+	lua_pushliteral(luaState, "editor");
+	lua_rawget(luaState, LUA_GLOBALSINDEX);
+	lua_pushcclosure(luaState, cf_scite_send, 1);
+	lua_rawset(luaState, -3);
+
+	lua_pushliteral(luaState, "SendOutput");
+	lua_pushliteral(luaState, "output");
+	lua_rawget(luaState, LUA_GLOBALSINDEX);
+	lua_pushcclosure(luaState, cf_scite_send, 1);
+	lua_rawset(luaState, -3);
+
+	lua_pushliteral(luaState, "ConstantName");
+	lua_pushcfunction(luaState, cf_scite_constname);
+	lua_rawset(luaState, -3);
+
+	lua_pushliteral(luaState, "Open");
+	lua_pushcfunction(luaState, cf_scite_open);
+	lua_rawset(luaState, -3);
+
+	lua_rawset(luaState, LUA_GLOBALSINDEX);
 
 	// Metatable for global namespace, to publish iface constants
 	if (luaL_newmetatable(luaState, "SciTE_MT_GlobalScope")) {
@@ -1197,16 +1443,20 @@ static bool InitGlobalScope(bool checkProperties) {
 	}
 	lua_setmetatable(luaState, LUA_GLOBALSINDEX);
 
-	if (checkProperties && resetMode != 0) {
+	if (checkProperties && reload) {
 		CheckStartupScript();
 	}
 
 	if (startupScript) {
-		int status = lua_dofile(luaState, startupScript);
+		// TODO: Should buffer be deactivated temporarily, so editor iface
+		// functions won't be available during a reset, just as they are not
+		// available during a normal startup?  Are there any other functions
+		// that should be blocked during startup, e.g. the ones that allow
+		// you to add or switch buffers?
 
-		if (status != 0) {
-			host->Trace("> Lua: error occurred in startup script\n");
-			// reset, or leave things in a possibly invalid state?
+		luaL_loadfile(luaState, startupScript);
+		if (!call_function(luaState, 0, true)) {
+			host->Trace(">Lua: error occurred while loading startup script\n");
 		}
 	}
 
@@ -1217,6 +1467,9 @@ static bool InitGlobalScope(bool checkProperties) {
 	lua_pushliteral(luaState, "SciTE_InitialState");
 	clone_table(luaState, LUA_GLOBALSINDEX, true);
 	lua_rawset(luaState, LUA_REGISTRYINDEX);
+	
+	IsolateGlobalReqLoadedTable();
+	PublishGlobalBufferData();
 
 	return true;
 }
@@ -1255,10 +1508,13 @@ bool LuaExtension::Finalise() {
 bool LuaExtension::Clear() {
 	if (luaState) {
 		InitGlobalScope(true);
-	} else if (CheckResetMode() && CheckStartupScript()) {
+		extensionScript.clear();
+	} else if ((GetPropertyInt("ext.lua.reset") >= 1) && CheckStartupScript()) {
 		InitGlobalScope(false);
 	}
-
+	if (luaState) {
+		CallNamedFunction("OnClear");
+	}
 	return false;
 }
 
@@ -1269,20 +1525,10 @@ bool LuaExtension::Load(const char *filename) {
 		int sl = strlen(filename);
 		if (sl >= 4 && strcmp(filename+sl-4, ".lua")==0) {
 			if (luaState || InitGlobalScope(false)) {
-				//host->Trace("> dofile ");
-				//host->Trace(filename);
-				//host->Trace("\n");
-
-				/* if (rawget_library_object(luaState,"dofile")) {
-					lua_pushstring(luaState, filename);
-					if (0 != lua_pcall(luaState, 1, 0, 0)) {
-						host->Trace("> Lua: error loading extension script\n");
-					}
-				} */
-
-				int status = lua_dofile(luaState, filename);
-				if (status != 0) {
-					host->Trace("> Lua: error loading extension script\n");
+				extensionScript = filename;
+				luaL_loadfile(luaState, filename);
+				if (!call_function(luaState, 0, true)) {
+					host->Trace(">Lua: error occurred while loading extension script\n");
 				}
 				loaded = true;
 			}
@@ -1291,6 +1537,85 @@ bool LuaExtension::Load(const char *filename) {
 	return loaded;
 }
 
+
+bool LuaExtension::InitBuffer(int index) {
+	//char msg[100];
+	//sprintf(msg, "InitBuffer(%d)\n", index);
+	//host->Trace(msg);
+
+	if (index > maxBufferIndex)
+		maxBufferIndex = index;
+
+	if (luaState) {
+		// This buffer might be recycled.  Clear the data associated
+		// with the old file.
+
+		lua_pushliteral(luaState, "SciTE_BufferData_Array");
+		lua_rawget(luaState, LUA_REGISTRYINDEX);
+		if (lua_istable(luaState, -1)) {
+			lua_pushnil(luaState);
+			lua_rawseti(luaState, -2, index);
+		}
+		// We also need to handle cases where Lua initialization is
+		// delayed (e.g. no startup script).  For that we'll just
+		// explicitly call InitBuffer(curBufferIndex)
+	}
+
+	curBufferIndex = index;
+
+	return false;
+}
+
+bool LuaExtension::ActivateBuffer(int index) {
+	//char msg[100];
+	//sprintf(msg, "ActivateBuffer(%d)\n", index);
+	//host->Trace(msg);
+
+	// Probably don't need to do anything with Lua here.  Setting
+	// curBufferIndex is important so that InitGlobalScope knows
+	// which buffer is active, in order to populate the 'buffer'
+	// global with the right data.
+
+	curBufferIndex = index;
+
+	return false;
+}
+
+bool LuaExtension::RemoveBuffer(int index) {
+	//char msg[100];
+	//sprintf(msg, "RemoveBuffer(%d)\n", index);
+	//host->Trace(msg);
+
+	if (luaState) {
+		// Remove the bufferdata element at index, and move
+		// the other elements down.  The element at the
+		// current maxBufferIndex can be discarded after
+		// it gets copied to maxBufferIndex-1.
+
+		lua_pushliteral(luaState, "SciTE_BufferData_Array");
+		lua_rawget(luaState, LUA_REGISTRYINDEX);
+		if (lua_istable(luaState, -1)) {
+			for (int i = index; i < maxBufferIndex; ++i) {
+				lua_rawgeti(luaState, -1, i+1);
+				lua_rawseti(luaState, -2, i);
+			}
+
+			lua_pushnil(luaState);
+			lua_rawseti(luaState, -2, maxBufferIndex);
+			
+			lua_pop(luaState, 1); // the bufferdata table
+		}
+	}
+
+	if (maxBufferIndex > 0)
+		maxBufferIndex--;
+
+	// Invalidate current buffer index; Activate or Init will follow.
+	curBufferIndex = -1;
+
+	return false;
+}
+
 bool LuaExtension::OnExecute(const char *s) {
 	bool handled = false;
 
@@ -1319,7 +1644,7 @@ bool LuaExtension::OnExecute(const char *s) {
 						lua_settop(luaState, stackBase+2);
 
 						if (!call_function(luaState, 1, true)) {
-							host->Trace("> Lua: error occurred while processing command\n");
+							host->Trace(">Lua: error occurred while processing command\n");
 						}
 					}
 				} else {
@@ -1351,15 +1676,16 @@ bool LuaExtension::OnSave(const char *filename) {
 	// should this be case insensitive on windows?  check for different
 	// ways of spelling a filename, e.g. short vs. long?
 	if (startupScript && 0 == strcmp(filename, startupScript)) {
-		char *autoReload = host->Property("ext.lua.auto.reload");
-		if (autoReload) {
-			if (autoReload[0] == '1') {
-				int status = lua_dofile(luaState, filename);
-				if (status != 0) {
-					host->Trace("> Lua: error reloading startup script\n");
-				}
+		if (GetPropertyInt("ext.lua.auto.reload") > 0) {
+			InitGlobalScope(false, true);
+			if (extensionScript.length()) {
+				Load(extensionScript.c_str());
 			}
-			delete [] autoReload;
+		}
+	} else if (extensionScript.length() && 0 == strcmp(filename, extensionScript.c_str())) {
+		if (GetPropertyInt("ext.lua.auto.reload") > 0) {
+			InitGlobalScope(false, false);
+			Load(extensionScript.c_str());
 		}
 	}
 
@@ -1387,6 +1713,10 @@ bool LuaExtension::OnMarginClick() {
 	return CallNamedFunction("OnMarginClick");
 }
 
+bool LuaExtension::OnUserListSelection(int listType, const char *selection) {
+	return CallNamedFunction("OnUserListSelection", listType, selection);
+}
+
 
 #ifdef _MSC_VER
 // Unreferenced inline functions are OK
