@@ -443,6 +443,8 @@ protected:
 	bool regExp;
 	bool wrapFind;
 	bool unSlash;
+	bool findInStyle;
+	int findStyle;
 	ComboMemory memFinds;
 	ComboMemory memReplaces;
 	ComboMemory memFiles;
@@ -518,6 +520,7 @@ protected:
 	bool topMost;
 	bool wrap;
 	bool wrapOutput;
+	int wrapStyle;
 	bool isReadOnly;
 	bool openFilesHere;
 	bool fullScreen;
@@ -633,6 +636,7 @@ protected:
 	sptr_t SendWindow(Window &w, unsigned int msg, uptr_t wParam=0, sptr_t lParam=0);
 	void SendChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t SendOutputEx(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0, bool direct = true);
+	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound=true);
 	int LengthDocument();
 	int GetCaretInLine();
 	void GetLine(char *text, int sizeText, int line=-1);
@@ -731,6 +735,7 @@ protected:
 	virtual void Find() = 0;
 	virtual int WindowMessageBox(Window &w, const SString &m, int style)=0;
 	void FindMessageBox(const SString &msg);
+	int FindInTarget(const char *findWhat, int lenFind, int startPosition, int endPosition);
 	int FindNext(bool reverseDirection, bool showWarnings = true);
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
@@ -875,6 +880,7 @@ protected:
 	virtual void ReadProperties();
 	void SetOneStyle(Window &win, int style, const StyleDefinition &sd);
 	void SetStyleFor(Window &win, const char *language);
+	void ReloadProperties();
 
 	void CheckReload();
 	void Activate(bool activeApp);
