@@ -183,6 +183,13 @@ void SciTEBase::SetDocumentAt(int index) {
 
 	buffers.SetCurrent(index);
 
+	if (extender) {
+		if (buffers.size > 1)
+			extender->ActivateBuffer(index);
+		else
+			extender->InitBuffer(0);
+	}
+
 	Buffer bufferNext = buffers.buffers[buffers.Current()];
 	isDirty = bufferNext.isDirty;
 	useMonoFont = bufferNext.useMonoFont;
@@ -235,8 +242,9 @@ void SciTEBase::SetDocumentAt(int index) {
 	CheckMenus();
 	UpdateStatusBar(true);
 
-	if (extender)
+	if (extender) {
 		extender->OnSwitchFile(fullPath);
+	}
 }
 
 void SciTEBase::UpdateBuffersCurrent() {
@@ -253,7 +261,6 @@ void SciTEBase::UpdateBuffersCurrent() {
 		buffers.buffers[currentbuf].unicodeMode = unicodeMode;
 
 		// retrieve fold state and store in buffer state info
-		//SendEditor(SCI_COLOURISE, 0, -1);
 		int maxLine = SendEditor(SCI_GETLINECOUNT);
 		int foldPoints = 0;
 
@@ -538,8 +545,12 @@ void SciTEBase::New() {
 	if ((buffers.length > 1) ||
 	        (buffers.Current() != 0) ||
 	        (buffers.buffers[0].isDirty) ||
-	        (!buffers.buffers[0].IsUntitled()))
-		buffers.SetCurrent(buffers.Add());
+			(!buffers.buffers[0].IsUntitled())) {
+			if (buffers.size == buffers.length) {
+				Close(false, false, true);
+			}
+			buffers.SetCurrent(buffers.Add());
+	}
 
 	sptr_t doc = GetDocumentAt(buffers.Current());
 	SendEditor(SCI_SETDOCPOINTER, 0, doc);
@@ -557,20 +568,22 @@ void SciTEBase::New() {
 	ClearDocument();
 	DeleteFileStackMenu();
 	SetFileStackMenu();
+	if (extender)
+		extender->InitBuffer(buffers.Current());
 }
 
-void SciTEBase::Close(bool updateUI, bool loadingSession) {
+void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew) {
 	bool closingLast = false;
 
 	if (buffers.size == 1) {
 		// With no buffer list, Close means close from MRU
 		closingLast = !(recentFileStack[0].IsSet());
-
 		buffers.buffers[0].Init();
 		buffers.buffers[0].useMonoFont = useMonoFont;
 		fullPath[0] = '\0';
 		ClearDocument(); //avoid double are-you-sure
-		StackMenu(0);
+		if (!makingRoomForNew)
+			StackMenu(0); // calls New, or Open, which calls InitBuffer
 	} else if (buffers.size > 1) {
 		if (buffers.Current() >= 0 && buffers.Current() < buffers.length) {
 			UpdateBuffersCurrent();
@@ -581,8 +594,13 @@ void SciTEBase::Close(bool updateUI, bool loadingSession) {
 		if (closingLast) {
 			buffers.buffers[0].Init();
 			buffers.buffers[0].useMonoFont = useMonoFont;
+			if (extender)
+				extender->InitBuffer(0);
 		} else {
+			extender->RemoveBuffer(buffers.Current());
 			buffers.RemoveCurrent();
+			if (extender && !makingRoomForNew)
+				extender->ActivateBuffer(buffers.Current());
 		}
 		Buffer bufferNext = buffers.buffers[buffers.Current()];
 		isDirty = bufferNext.isDirty;
@@ -626,6 +644,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession) {
 			DisplayAround(bufferNext);
 		}
 	}
+
 	if (updateUI) {
 		BuffersMenu();
 		UpdateStatusBar(true);
@@ -1235,8 +1254,9 @@ void SciTEBase::ToolsMenu(int item) {
 	}
 }
 
-int DecodeMessage(char *cdoc, char *sourcePath, int format) {
+int DecodeMessage(char *cdoc, char *sourcePath, int format, int &column) {
 	sourcePath[0] = '\0';
+	column= -1; // default to not detected
 	switch (format) {
 	case SCE_ERR_PYTHON: {
 			// Python
@@ -1255,18 +1275,17 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format) {
 			return sourceNumber;
 		}
 	case SCE_ERR_GCC: {
-			// GCC - look for number followed by colon to be line number
-			// This will be preceded by file name
+			// GCC - look for number after colon to be line number
+			// This will be preceded by file name.
+			// Lua debug traceback messages also piggyback this style, but begin with a tab.
+			if (cdoc[0] == '\t')
+				++cdoc;
 			for (int i = 0; cdoc[i]; i++) {
-				if (isdigit(cdoc[i]) && cdoc[i + 1] == ':') {
-					int j = i;
-					while (j > 0 && isdigit(cdoc[j - 1])) {
-						j--;
-					}
-					int sourceNumber = atoi(cdoc + j) - 1;
-					if (j > 0) {
-						strncpy(sourcePath, cdoc, j - 1);
-						sourcePath[j - 1] = 0;
+				if (cdoc[i] == ':' && isdigit(cdoc[i + 1])) {
+					int sourceNumber = atoi(cdoc + i + 1) - 1;
+					if (i > 0) {
+						strncpy(sourcePath, cdoc, i);
+						sourcePath[i] = 0;
 					}
 					return sourceNumber;
 				}
@@ -1502,6 +1521,46 @@ int DecodeMessage(char *cdoc, char *sourcePath, int format) {
 			}
 			break;
 		}
+
+	case SCE_ERR_TIDY: {
+			/* HTML Tidy error/warnings look like:
+			 * line 8 column 1 - Error: unexpected </head> in <meta>
+			 * line 41 column 1 - Warning: <table> lacks "summary" attribute
+			 */
+			char *line = strchr(cdoc, ' ');
+			if (line) {
+				char *col = strchr(line+1, ' ');
+				if (col) {
+					*col = '\0';
+					int lnr = atoi(line) - 1;
+					col = strchr(col+1, ' ');
+					if (col) {
+						char *endcol = strchr(col+1, ' ');
+						if (endcol) {
+							*endcol = '\0';
+							column = atoi(col)-1;
+							return lnr;
+						}
+					}
+				}
+			}
+			break;
+		}
+
+	case SCE_ERR_JAVA_STACK: {
+			/* Java runtime stack trace
+				\tat <methodname>(<filename>:<line>)
+				 */
+			char *startPath = strchr(cdoc, '(') + 1;
+			char *endPath = strchr(startPath, ':');
+			int length = endPath - startPath;
+			if (length > 0) {
+				strncpy(sourcePath, startPath, length);
+				sourcePath[length] = 0;
+				int sourceNumber = atoi(endPath+1) - 1;
+				return sourceNumber;
+			}
+		}
 	}	// switch
 	return - 1;
 }
@@ -1540,9 +1599,10 @@ void SciTEBase::GoMessage(int dir) {
 			if (!cdoc)
 				return;
 			GetRange(wOutput, startPosLine, startPosLine + lineLength, cdoc);
+			SString message(cdoc);
 			char sourcePath[MAX_PATH];
-			int sourceLine = DecodeMessage(cdoc, sourcePath, style);
-			//printf("<%s> %d %d\n",sourcePath, sourceLine, lookLine);
+			int column;
+			int sourceLine = DecodeMessage(cdoc, sourcePath, style, column);
 			//Platform::DebugPrintf("<%s> %d %d\n",sourcePath, sourceLine, lookLine);
 			if (sourceLine >= 0) {
 				if (0 != strcmp(sourcePath, fileName)) {
@@ -1593,8 +1653,23 @@ void SciTEBase::GoMessage(int dir) {
 				           "error.marker.back", ColourDesired(0xff, 0xff, 0)));
 				SendEditor(SCI_MARKERADD, sourceLine, 0);
 				int startSourceLine = SendEditor(SCI_POSITIONFROMLINE, sourceLine, 0);
+				int endSourceline= SendEditor(SCI_POSITIONFROMLINE, sourceLine+1, 0);
+				if (column >= 0) {
+					// Get the position in line according to current tab setting
+					startSourceLine = SendEditor(SCI_FINDCOLUMN, sourceLine, column); 
+				}
 				EnsureRangeVisible(startSourceLine, startSourceLine);
-				SetSelection(startSourceLine, startSourceLine);
+				if (props.GetInt("error.select.line")==1) {
+					//select whole source source line from column with error
+					SetSelection(endSourceline, startSourceLine);
+				} else {
+					//simply move cursor to line, don't do any selection
+					SetSelection(startSourceLine, startSourceLine);
+				}
+				message.substitute('\t', ' ');
+				message.remove("\n");
+				props.Set("CurrentMessage", message.c_str());
+				UpdateStatusBar(false);
 				WindowSetFocus(wEditor);
 			}
 			delete []cdoc;
