@@ -365,6 +365,9 @@ static UniMode CookieValue(const SString &s) {
 		posCoding += static_cast<int>(strlen(codingCookie));
 		if ((s[posCoding] == ':') || (s[posCoding] == '=')) {
 			posCoding++;
+			if ((s[posCoding] == '\"') || (s[posCoding] == '\'')) {
+				posCoding++;
+			}
 			while ((posCoding < static_cast<int>(s.length())) &&
 				(isSpaceChar(s[posCoding]))) {
 				posCoding++;
@@ -384,28 +387,94 @@ static UniMode CookieValue(const SString &s) {
 	return uni8Bit;
 }
 
-void SciTEBase::OpenFile(bool suppressMessage) {
+void SciTEBase::DiscoverEOLSetting() {
+	int linesCR;
+	int linesLF;
+	int linesCRLF;
+	SetEol();
+	CountLineEnds(linesCR, linesLF, linesCRLF);
+	if (((linesLF >= linesCR) && (linesLF > linesCRLF)) || ((linesLF > linesCR) && (linesLF >= linesCRLF)))
+		SendEditor(SCI_SETEOLMODE, SC_EOL_LF);
+	else if (((linesCR >= linesLF) && (linesCR > linesCRLF)) || ((linesCR > linesLF) && (linesCR >= linesCRLF)))
+		SendEditor(SCI_SETEOLMODE, SC_EOL_CR);
+	else if (((linesCRLF >= linesLF) && (linesCRLF > linesCR)) || ((linesCRLF > linesLF) && (linesCRLF >= linesCR)))
+		SendEditor(SCI_SETEOLMODE, SC_EOL_CRLF);
+}
+
+void SciTEBase::DiscoverIndentSetting() {
+	int lengthDoc = LengthDocument();
+	WindowAccessor acc(wEditor.GetID(), props);
+	bool newline = true;
+	int indent = 0; // current line indentation
+	int tabSizes[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 }; // number of lines with corresponding indentation (index 0 - tab)
+	int prevIndent = 0; // previous line indentation
+	int prevTabSize = -1; // previous line tab size
+	for (int i = 0; i < lengthDoc; i++) {
+		char ch = acc[i];
+		if (ch == '\r' || ch == '\n') {
+			indent = 0;
+			newline = true;
+		} else if (newline && ch == ' ') {
+			indent++;
+		} else if (newline) {
+			if (indent) {
+				if (indent == prevIndent && prevTabSize != -1) {
+					tabSizes[prevTabSize]++;
+				} else if (indent > prevIndent && prevIndent != -1) {
+					if (indent - prevIndent <= 8) {
+						prevTabSize = indent - prevIndent;
+						tabSizes[prevTabSize]++;
+					} else {
+						prevTabSize = -1;
+					}
+				}
+				prevIndent = indent;
+			} else if (ch == '\t') {
+				tabSizes[0]++;
+				prevIndent = -1;
+			} else {
+				prevIndent = 0;
+			}
+			newline = false;
+		}
+	}
+	// maximum non-zero indent
+	int topTabSize = -1;
+	for (int j = 0; j <= 8; j++) {
+		if (tabSizes[j] && (topTabSize == -1 || tabSizes[j] > tabSizes[topTabSize])) {
+			topTabSize = j;
+		}
+	}
+	// set indentation
+	if (topTabSize == 0) {
+		SendEditor(SCI_SETUSETABS, 1);
+	} else if (topTabSize != -1) {
+		SendEditor(SCI_SETUSETABS, 0);
+		SendEditor(SCI_SETINDENT, topTabSize);
+	}
+}
+
+void SciTEBase::OpenFile(int fileSize, bool suppressMessage) {
 	Utf8_16_Read convert;
 
 	FILE *fp = fopen(fullPath, fileRead);
 	if (fp) {
-		fseek(fp, 0, SEEK_END);
-		int allocation = ftell(fp);
-		SendEditor(SCI_ALLOCATE, allocation + 1000);
-		fseek(fp, 0, SEEK_SET);
 		fileModTime = GetModTime(fullPath);
 		fileModLastAsk = fileModTime;
+		SendEditor(SCI_BEGINUNDOACTION);	// Group together clear and insert
 		SendEditor(SCI_CLEARALL);
 		char data[blockSize];
 		size_t lenFile = fread(data, 1, sizeof(data), fp);
 		SString l1 = ExtractLine(data, lenFile);
 		SString l2 = ExtractLine(data+l1.length(), lenFile-l1.length());
+		SendEditor(SCI_ALLOCATE, fileSize + 1000);
 		while (lenFile > 0) {
 			lenFile = convert.convert(data, lenFile);
 			SendEditorString(SCI_ADDTEXT, lenFile, convert.getNewBuf());
 			lenFile = fread(data, 1, sizeof(data), fp);
 		}
 		fclose(fp);
+		SendEditor(SCI_ENDUNDOACTION);
 		unicodeMode = static_cast<UniMode>(
 			static_cast<int>(convert.getEncoding()));
 		// Check the first two lines for coding cookies
@@ -424,18 +493,13 @@ void SciTEBase::OpenFile(bool suppressMessage) {
 		SendEditor(SCI_SETCODEPAGE, codePage);
 
 		if (props.GetInt("eol.auto")) {
-			int linesCR;
-			int linesLF;
-			int linesCRLF;
-			SetEol();
-			CountLineEnds(linesCR, linesLF, linesCRLF);
-			if (((linesLF >= linesCR) && (linesLF > linesCRLF)) || ((linesLF > linesCR) && (linesLF >= linesCRLF)))
-				SendEditor(SCI_SETEOLMODE, SC_EOL_LF);
-			else if (((linesCR >= linesLF) && (linesCR > linesCRLF)) || ((linesCR > linesLF) && (linesCR >= linesCRLF)))
-				SendEditor(SCI_SETEOLMODE, SC_EOL_CR);
-			else if (((linesCRLF >= linesLF) && (linesCRLF > linesCR)) || ((linesCRLF > linesLF) && (linesCRLF >= linesCR)))
-				SendEditor(SCI_SETEOLMODE, SC_EOL_CRLF);
+			DiscoverEOLSetting();
 		}
+
+		if (props.GetInt("indent.auto")) {
+			DiscoverIndentSetting();
+		}
+
 	} else if (!suppressMessage) {
 		SString msg = LocaliseMessage("Could not open file '^0'.", fullPath);
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
@@ -457,12 +521,27 @@ bool SciTEBase::PreOpenCheck(const char *) {
 	return false;
 }
 
+static int GetFileLength(const char *filePath) {
+	int size = -1;
+	if (filePath && *filePath) {
+		FILE *fp = fopen(filePath, fileRead);
+		if (fp) {
+			fseek(fp, 0, SEEK_END);
+			size = ftell(fp);
+			fseek(fp, 0, SEEK_SET);
+			fclose(fp);
+		}
+	}
+	return size;
+}
+
 bool SciTEBase::Open(const char *file, OpenFlags of) {
 	InitialiseBuffers();
 
 	if (!file) {
 		SString msg = LocaliseMessage("Bad file.");
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		return false;
 	}
 #ifdef __vms
 	static char fixedFileName[MAX_PATH];
@@ -485,9 +564,29 @@ bool SciTEBase::Open(const char *file, OpenFlags of) {
 		return false;
 	}
 
+	int size = 0;
+	if (absPath[0] != '\0') {
+		// Real file, not empty buffer
+		size = GetFileLength(absPath);
+		int maxSize = props.GetInt("max.file.size");
+		if (maxSize > 0 && size > maxSize) {
+			SString sSize(size), sMaxSize(maxSize);
+			SString msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
+						      "larger than the ^2 bytes limit set in the properties.\n"
+						      "Do you still want to open it?",
+						      absPath, sSize.c_str(), sMaxSize.c_str());
+			int answer = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
+			if (answer != IDYES) {
+				return false;
+			}
+		}
+	}
+
 	if (buffers.size == buffers.length) {
 		AddFileToStack(fullPath, GetSelection(), GetCurrentScrollPosition());
 		ClearDocument();
+		if (extender)
+			extender->InitBuffer(buffers.Current());
 	} else {
 		if (index < 0 || !(of & ofForceLoad)) { // No new buffer, already opened
 			New();
@@ -511,7 +610,7 @@ bool SciTEBase::Open(const char *file, OpenFlags of) {
 			SendEditor(SCI_SETUNDOCOLLECTION, 0);
 		}
 
-		OpenFile(of & ofQuiet);
+		OpenFile(size, of & ofQuiet);
 
 		if (of & ofPreserveUndo) {
 			SendEditor(SCI_ENDUNDOACTION);
@@ -525,6 +624,8 @@ bool SciTEBase::Open(const char *file, OpenFlags of) {
 	DeleteFileStackMenu();
 	SetFileStackMenu();
 	SetWindowName();
+	if (lineNumbers && lineNumbersExpand)
+		SetLineNumberWidth();
 	UpdateStatusBar(true);
 	if (extender)
 		extender->OnOpen(fullPath);
@@ -649,7 +750,7 @@ bool SciTEBase::OpenSelected() {
 
 void SciTEBase::Revert() {
 	RecentFile rf = GetFilePosition();
-	OpenFile(false);
+	OpenFile(GetFileLength(fullPath), false);
 	DisplayAround(rf);
 }
 
@@ -853,6 +954,17 @@ bool SciTEBase::SaveBuffer(const char *saveName) {
 	return retVal;
 }
 
+void SciTEBase::ReloadProperties() {
+	ReadGlobalPropFile();
+	SetImportMenu();
+	ReadLocalPropFile();
+	ReadAbbrevPropFile();
+	ReadProperties();
+	SetWindowName();
+	BuffersMenu();
+	Redraw();
+}
+
 // Returns false if cancelled or failed to save
 bool SciTEBase::Save() {
 	if (fileName[0]) {
@@ -864,14 +976,7 @@ bool SciTEBase::Save() {
 			fileModTime = GetModTime(fullPath);
 			SendEditor(SCI_SETSAVEPOINT);
 			if (IsPropertiesFile(fileName)) {
-				ReadGlobalPropFile();
-				SetImportMenu();
-				ReadLocalPropFile();
-				ReadAbbrevPropFile();
-				ReadProperties();
-				SetWindowName();
-				BuffersMenu();
-				Redraw();
+				ReloadProperties();
 			}
 		} else {
 			SString msg = LocaliseMessage(
