@@ -7,6 +7,63 @@
 
 #include "SciTEWin.h"
 
+/**
+ * Set up properties for FileTime, FileDate, CurrentTime, CurrentDate and FileAttr.
+ */
+void SciTEWin::SetFileProperties(
+    PropSet &ps) {			///< Property set to update.
+
+	const int TEMP_LEN = 100;
+	char temp[TEMP_LEN];
+	HANDLE hf = ::CreateFile(fullPath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+	if (hf != INVALID_HANDLE_VALUE) {
+		FILETIME ft;
+		::GetFileTime(hf, NULL, NULL, &ft);
+		::CloseHandle(hf);
+		FILETIME lft;
+		::FileTimeToLocalFileTime(&ft, &lft);
+		SYSTEMTIME st;
+		::FileTimeToSystemTime(&lft, &st);
+		::GetTimeFormat(LOCALE_USER_DEFAULT,
+		                0, &st,
+		                NULL, temp, TEMP_LEN);
+		ps.Set("FileTime", temp);
+
+		::GetDateFormat(LOCALE_USER_DEFAULT,
+		                DATE_SHORTDATE, &st,
+		                NULL, temp, TEMP_LEN);
+		ps.Set("FileDate", temp);
+
+		DWORD attr = ::GetFileAttributes(fullPath);
+		SString fa;
+		if (attr & FILE_ATTRIBUTE_READONLY) {
+			fa += "R";
+		}
+		if (attr & FILE_ATTRIBUTE_HIDDEN) {
+			fa += "H";
+		}
+		if (attr & FILE_ATTRIBUTE_SYSTEM) {
+			fa += "S";
+		}
+		ps.Set("FileAttr", fa.c_str());
+	} else {
+		/* Reset values for new buffers with no file */
+		ps.Set("FileTime", "");
+		ps.Set("FileDate", "");
+		ps.Set("FileAttr", "");
+	}
+
+	::GetDateFormat(LOCALE_USER_DEFAULT,
+	                DATE_SHORTDATE, NULL,     	// Current date
+	                NULL, temp, TEMP_LEN);
+	ps.Set("CurrentDate", temp);
+
+	::GetTimeFormat(LOCALE_USER_DEFAULT,
+	                0, NULL,     	// Current time
+	                NULL, temp, TEMP_LEN);
+	ps.Set("CurrentTime", temp);
+}
+
 /**
  * Update the status bar text.
  */
@@ -29,6 +86,58 @@ void SciTEWin::Notify(SCNotification *notification) {
 		}
 		break;
 
+	case NM_RCLICK:
+		// Right click on a control
+		if (notification->nmhdr.idFrom == IDM_TABWIN) {
+
+			Point ptCursor;
+			::GetCursorPos(reinterpret_cast<POINT *>(&ptCursor));
+			Point ptClient = ptCursor;
+			::ScreenToClient(reinterpret_cast<HWND>(wTabBar.GetID()),
+			                 reinterpret_cast<POINT *>(&ptClient));
+			TCHITTESTINFO info;
+			info.pt.x = ptClient.x;
+			info.pt.y = ptClient.y;
+
+			int tabbarHitLast = TabCtrl_HitTest(reinterpret_cast<HWND> (wTabBar.GetID()), &info);
+
+			if (buffers.Current() != tabbarHitLast) {
+				SetDocumentAt(tabbarHitLast);
+				CheckReload();
+			}
+
+			// Pop up menu here:
+			popup.CreatePopUp();
+			AddToPopUp("Close", IDM_CLOSE, true);
+			AddToPopUp("");
+			AddToPopUp("Save", IDM_SAVE, true);
+			AddToPopUp("Save As", IDM_SAVEAS, true);
+			AddToPopUp("");
+
+			bool bAddSeparator = false;
+			for (int item = 0; item < toolMax; item++) {
+				int itemID = IDM_TOOLS + item;
+				SString prefix = "command.name.";
+				prefix += SString(item);
+				prefix += ".";
+				SString commandName = props.GetNewExpand(prefix.c_str(), fileName);
+				if (commandName.length()) {
+					SString sMenuItem = commandName;
+					SString sMnemonic = "Ctrl+";
+					sMnemonic += SString(item);
+					AddToPopUp(sMenuItem.c_str(), itemID, true);
+					bAddSeparator = true;
+				}
+			}
+
+			if (bAddSeparator)
+				AddToPopUp("");
+
+			AddToPopUp("Print", IDM_PRINT, true);
+			popup.Show(ptCursor, wSciTE);
+		}
+		break;
+
 	case NM_CLICK:
 		// Click on a control
 		if (notification->nmhdr.idFrom == IDM_STATUSWIN) {
@@ -268,7 +377,7 @@ void SciTEWin::SizeSubWindows() {
 
 
 
-// Keymod param is interpreted using the same notation (and much the same 
+// Keymod param is interpreted using the same notation (and much the same
 // code) as KeyMatch uses in SciTEWin.cxx.
 
 
@@ -284,8 +393,8 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 		keycode = SciTEKeys::ParseKeyCode(mnemonic);
 		if (keycode) {
 			sTextMnemonic += "\t";
-		sTextMnemonic += LocaliseAccelerator(mnemonic, itemID);
-	}
+			sTextMnemonic += LocaliseAccelerator(mnemonic, itemID);
+		}
 		// the keycode could be used to make a custom accelerator table
 		// but for now, the menu's item data is used instead for command
 		// tools, and for other menu entries it is just discarded.
@@ -484,8 +593,10 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 					}
 					text = LocaliseString(text.c_str(), true);
 					if (text.length()) {
-						text += "\t";
-						text += LocaliseAccelerator(accel.c_str(), mii.wID);
+						if (accel != "") {
+							text += "\t";
+							text += LocaliseAccelerator(accel.c_str(), mii.wID);
+						}
 						mii.dwTypeData = const_cast<char *>(text.c_str());
 						::SetMenuItemInfo(hmenu, i, TRUE, &mii);
 					}
@@ -724,5 +835,12 @@ void SciTEWin::Creation() {
 	::SendMessage(reinterpret_cast<HWND>(wStatusBar.GetID()),
 	              SB_SETPARTS, 1,
 	              reinterpret_cast<LPARAM>(widths));
+
+#ifndef NO_LUA
+		if (props.GetExpanded("ext.lua.startup.script").length() == 0)
+			DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
+#else
+		DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
+#endif
 }
 
